#pragma once
#include <vector>
#include <map>
#include "Node.h"
#include "TextRenderer.h"
#include "FreeCameraComponent.h"
#include "FollowCameraComponent.h"
#include "Scene.h"
#include "MainRenderer.h"
#include "PickRenderer.h"
#include "PostEffectRenderer.h"
#include "SnowParticleSystemComponent.h"
#include <irrKlang.h>

class World
{
public:
	World(int windowWidth, int windowHeight);
	~World();

	bool Initialize();

	void Update(double curFrame, double deltaFrame);

	void Render();

	//нڵnameظIDΨһ
	Node* AddNode(const stringT& name);

	//ɾڵ㣬Ƚڵ㼰ӽڵƶm_NodesToDestroyһ֡ͳһɾ
	bool RemoveNode(Node* node);

	//ɾһڵ
	bool RemoveNode(unsigned long id);

	//ҵеһƥĽڵ
	Node* FindNode(const stringT& name);

	//óڵ
	Node* GetRootNode();

	//ʰȡ
	Node* PickNode(unsigned int x, unsigned int y);

	//UI¼
	void OnMouseMove(double xPos, double yPos);

	void OnMouseDown();

	void OnMouseUp();

	void OnKeyboard(int key);

	void OnMouseScroll(double xOffset, double yOffset);

	void OnResize(int width, int height);

	void ToFree(const glm::vec3& position);

	void ToFollow(Node* node);

	void ToFlyMode(bool value);
	bool& GetFlyMode();

	CameraComponent* GetActiveCamera();

	CameraComponent* GetFreeCamera();

	CameraComponent* GetFollowCamera();

	Scene* GetScene();

	float GetHeightAt(const glm::vec3& position);

	bool GetEnablePostEffect();
	void SetEnablePostEffect(bool value);

	bool& GetEnableLight();
	void SetEnableLight(bool value);

	bool& GetEnableSkybox();
	void SetEnableSkybox(bool value);

	bool& GetEnableRenderNormal();
	void SetEnableRenderNormal(bool value);

	bool& GetEnableSnow();
	void SetEnableSnow(bool value);

	bool& GetEnableBoundingBox();
	void SetEnableBoundingBox(bool value);

	int GetWindowWidth();
	int GetWindowHeight();

	float& GetFreeCamKeySensitivity();

	glm::vec3& GetSunDirection();
	void SetSunDirection(const glm::vec3& value);

	float& GetWaterHeight();

	void SetBGM(std::string path);
	void SetBGMVolume(float value);
	void PlayPauseBGM(bool play);
	bool GetBGMPlayPause();

private:
	std::map<unsigned long, Node*> m_Nodes;//еĽڵ㣬ʹmapڲѯ
	std::vector<Node*> m_NodesToDestroy;//ҪɾĽڵ㣬һһ֡ɾ
	Node* m_RootNode;//ڵ
	Node* m_CameraNode;//Ľڵ
	unsigned long m_CurNodeID;//ڵid 1ʼ
	FreeCameraComponent* m_FreeCamera;
	FollowCameraComponent* m_FollowCamera;
	int m_WindowWidth;
	int m_WindowHeight;
	float m_FPS;

	TextRenderer* m_TextRender;

	Node* m_PickingNode;
	SRTTransformComponent* m_PickingNodeSRT;

	Scene* m_Scene;
	MainRenderer* m_Renderer;

	double m_CurrentFrame;
	double m_DeltaFrame;

	bool m_IsDragging;
	float m_PickNodeScreenZ;

	SnowParticleSystemComponent* m_SnowCmp;
	irrklang::ISound* m_BGM;
	bool m_BGMPlaying;
};

